import type_enforced


@type_enforced.Enforcer
def my_fn(a: int, b: [int, str] = 2, c: int = 3) -> None:
    return 1


success = False

try:
    my_fn(a=1, b=2, c=3)  # Error (return is not None)
except Exception as e:
    if "Type mismatch" in str(e):
        success = True

if success:
    print("test_fn_02.py passed")
else:
    print("test_fn_02.py failed")



import type_enforced
from dataclasses import dataclass


@type_enforced.Enforcer
@dataclass
class Foo:
    bar: int
    baz: str


passed = True
try:
    foo = Foo(bar=1, baz="a")
except:
    passed = False

try:
    foo = Foo(bar="a", baz=1)
    passed = False
except:
    pass

if passed:
    print("test_class_11.py passed")
else:
    print("test_class_11.py failed")



import type_enforced
from typing import Type


class Foo:
    def __init__(self) -> None:
        pass


class Bar(Foo):
    def __init__(self) -> None:
        super().__init__()


@type_enforced.Enforcer
class Baz:
    def __init__(self, use_class: [Type[Foo]]) -> None:
        self.object = use_class


success = True
try:
    x = Baz(Foo)
except:
    success = False

try:
    y = Baz(Bar)
    success = False
except:
    pass

if success:
    print("test_class_04.py passed")
else:
    print("test_class_04.py failed")



import type_enforced


@type_enforced.Enforcer
def my_fn(
    a: list[str], b: dict[int], c: tuple[int, float], d: set[str]
) -> None:
    return None


success_1 = True
try:
    my_fn(a=["a"], b={"a": 1}, c=(1, 1.5), d={"a"})
except:
    success_1 = False

success_2 = False
try:
    my_fn(
        a=[1], b={"a": 1}, c=(1, 1.5), d={"a"}
    )  # Error (a can only accept str)
except Exception as e:
    if "Type mismatch" in str(e):
        success_2 = True

success_3 = False
try:
    my_fn(
        a=["a"], b={"a": "a"}, c=(1, 1.5), d={"a"}
    )  # Error (b can only accept int)
except Exception as e:
    if "Type mismatch" in str(e):
        success_3 = True

if success_1 and success_2 and success_3:
    print("test_fn_07.py passed")
else:
    print("test_fn_07.py failed")



import type_enforced
import sys


class Foo:
    @type_enforced.Enforcer
    @classmethod
    def add(self, a: int, b: int) -> int:
        return a + b

    @type_enforced.Enforcer
    @staticmethod
    def subtract(a: int, b: int) -> int:
        return a - b


success = True

try:
    x = Foo.add(1, 2)
    y = Foo.subtract(4, 3)
    success = success and x == 3 and y == 1
except:
    success = False

try:
    Foo.add(1, 2.0)
    success = False
except:
    pass

try:
    Foo.subtract(1, 2.0)
    success = False
except:
    pass

# classmethod and staticmethod wrappers do not contain annotations prior to 3.9
if success or sys.version_info <= (3, 10, 0):
    print("test_class_06.py passed")
else:
    print("test_class_06.py failed")



import type_enforced


@type_enforced.Enforcer()
def my_fn(a: int):
    return None


@type_enforced.Enforcer(enabled=False)
def my_fn2(a: int):
    return None


success = True

try:
    my_fn(a=1)
    my_fn2(a=1)
    my_fn2(a="1")
except:
    success = False

try:
    my_fn(a="1")
    success = False
except:
    pass

if success:
    print("test_fn_15.py passed")
else:
    print("test_fn_15.py failed")



import type_enforced
from typing import Literal


@type_enforced.Enforcer
def my_fn(a: Literal["a", "b"]):
    pass


success_1 = True
try:
    my_fn(a="a")
    my_fn(a="b")
except:
    success_1 = False

success_2 = False
try:
    my_fn(a="c")
except Exception as e:
    if "Literal validation error" in str(e):
        success_2 = True

if success_1 and success_2:
    print("test_fn_13.py passed")
else:
    print("test_fn_13.py failed")



from test_class_12_utils.bar import Bar, foo
from test_class_12_utils.baz import Baz

passed = True

# Special Test for delayed class inheritance binding

try:
    foo(Baz())
    foo(Bar())
except:
    passed = False

try:
    foo(1)
    passed = False
except:
    pass

if passed:
    print("test_class_12.py passed")
else:
    print("test_class_12.py failed")
